/*******************************************************************************
 * Copyright (c) 2006, 2020 Borland Software Corporation, CEA LIST, ARTAL
 * 
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License 2.0
 *  which accompanies this distribution, and is available at
 *  https://www.eclipse.org/legal/epl-2.0/
 * 
 *  SPDX-License-Identifier: EPL-2.0
 * 
 *  Contributors:
 *     Borland - initial API and implementation
 *     Aurelien Didier (ARTAL) - aurelien.didier51@gmail.com - Bug 569174
 ******************************************************************************/
/**
 * <copyright>
 * </copyright>
 *
 * $Id$
 */
package org.eclipse.papyrus.gmf.gmfgraph.impl;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.impl.EFactoryImpl;
import org.eclipse.emf.ecore.plugin.EcorePlugin;
import org.eclipse.papyrus.gmf.gmfgraph.Alignment;
import org.eclipse.papyrus.gmf.gmfgraph.AlignmentFacet;
import org.eclipse.papyrus.gmf.gmfgraph.BasicFont;
import org.eclipse.papyrus.gmf.gmfgraph.BorderLayout;
import org.eclipse.papyrus.gmf.gmfgraph.BorderLayoutData;
import org.eclipse.papyrus.gmf.gmfgraph.BorderRef;
import org.eclipse.papyrus.gmf.gmfgraph.Canvas;
import org.eclipse.papyrus.gmf.gmfgraph.CenterLayout;
import org.eclipse.papyrus.gmf.gmfgraph.ChildAccess;
import org.eclipse.papyrus.gmf.gmfgraph.ColorConstants;
import org.eclipse.papyrus.gmf.gmfgraph.ColorPin;
import org.eclipse.papyrus.gmf.gmfgraph.Compartment;
import org.eclipse.papyrus.gmf.gmfgraph.CompoundBorder;
import org.eclipse.papyrus.gmf.gmfgraph.Connection;
import org.eclipse.papyrus.gmf.gmfgraph.ConstantColor;
import org.eclipse.papyrus.gmf.gmfgraph.CustomAttribute;
import org.eclipse.papyrus.gmf.gmfgraph.CustomBorder;
import org.eclipse.papyrus.gmf.gmfgraph.CustomConnection;
import org.eclipse.papyrus.gmf.gmfgraph.CustomDecoration;
import org.eclipse.papyrus.gmf.gmfgraph.CustomFigure;
import org.eclipse.papyrus.gmf.gmfgraph.CustomLayout;
import org.eclipse.papyrus.gmf.gmfgraph.CustomLayoutData;
import org.eclipse.papyrus.gmf.gmfgraph.CustomPin;
import org.eclipse.papyrus.gmf.gmfgraph.DefaultSizeFacet;
import org.eclipse.papyrus.gmf.gmfgraph.DiagramLabel;
import org.eclipse.papyrus.gmf.gmfgraph.Dimension;
import org.eclipse.papyrus.gmf.gmfgraph.Direction;
import org.eclipse.papyrus.gmf.gmfgraph.Ellipse;
import org.eclipse.papyrus.gmf.gmfgraph.FigureAccessor;
import org.eclipse.papyrus.gmf.gmfgraph.FigureDescriptor;
import org.eclipse.papyrus.gmf.gmfgraph.FigureGallery;
import org.eclipse.papyrus.gmf.gmfgraph.FigureRef;
import org.eclipse.papyrus.gmf.gmfgraph.FlowLayout;
import org.eclipse.papyrus.gmf.gmfgraph.FontStyle;
import org.eclipse.papyrus.gmf.gmfgraph.GMFGraphFactory;
import org.eclipse.papyrus.gmf.gmfgraph.GMFGraphPackage;
import org.eclipse.papyrus.gmf.gmfgraph.GeneralFacet;
import org.eclipse.papyrus.gmf.gmfgraph.GradientFacet;
import org.eclipse.papyrus.gmf.gmfgraph.GridLayout;
import org.eclipse.papyrus.gmf.gmfgraph.GridLayoutData;
import org.eclipse.papyrus.gmf.gmfgraph.Insets;
import org.eclipse.papyrus.gmf.gmfgraph.InvisibleRectangle;
import org.eclipse.papyrus.gmf.gmfgraph.Label;
import org.eclipse.papyrus.gmf.gmfgraph.LabelOffsetFacet;
import org.eclipse.papyrus.gmf.gmfgraph.LabeledContainer;
import org.eclipse.papyrus.gmf.gmfgraph.LayoutRef;
import org.eclipse.papyrus.gmf.gmfgraph.LineBorder;
import org.eclipse.papyrus.gmf.gmfgraph.LineKind;
import org.eclipse.papyrus.gmf.gmfgraph.MarginBorder;
import org.eclipse.papyrus.gmf.gmfgraph.Node;
import org.eclipse.papyrus.gmf.gmfgraph.Point;
import org.eclipse.papyrus.gmf.gmfgraph.Polygon;
import org.eclipse.papyrus.gmf.gmfgraph.PolygonDecoration;
import org.eclipse.papyrus.gmf.gmfgraph.Polyline;
import org.eclipse.papyrus.gmf.gmfgraph.PolylineConnection;
import org.eclipse.papyrus.gmf.gmfgraph.PolylineDecoration;
import org.eclipse.papyrus.gmf.gmfgraph.RGBColor;
import org.eclipse.papyrus.gmf.gmfgraph.Rectangle;
import org.eclipse.papyrus.gmf.gmfgraph.Rectangle2D;
import org.eclipse.papyrus.gmf.gmfgraph.RoundedRectangle;
import org.eclipse.papyrus.gmf.gmfgraph.SVGFigure;
import org.eclipse.papyrus.gmf.gmfgraph.SVGProperty;
import org.eclipse.papyrus.gmf.gmfgraph.SVGPropertyType;
import org.eclipse.papyrus.gmf.gmfgraph.ScalablePolygon;
import org.eclipse.papyrus.gmf.gmfgraph.StackLayout;
import org.eclipse.papyrus.gmf.gmfgraph.VerticalLabel;
import org.eclipse.papyrus.gmf.gmfgraph.VisiblePin;
import org.eclipse.papyrus.gmf.gmfgraph.XYLayout;
import org.eclipse.papyrus.gmf.gmfgraph.XYLayoutData;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model <b>Factory</b>.
 * <!-- end-user-doc -->
 * @generated
 */
public class GMFGraphFactoryImpl extends EFactoryImpl implements GMFGraphFactory {
	/**
	 * Creates the default factory implementation.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public static GMFGraphFactory init() {
		try {
			GMFGraphFactory theGMFGraphFactory = (GMFGraphFactory)EPackage.Registry.INSTANCE.getEFactory("http://www.eclipse.org/papyrus/gmf/2020/GraphicalDefinition"); 
			if (theGMFGraphFactory != null) {
				return theGMFGraphFactory;
			}
		}
		catch (Exception exception) {
			EcorePlugin.INSTANCE.log(exception);
		}
		return new GMFGraphFactoryImpl();
	}

	/**
	 * Creates an instance of the factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public GMFGraphFactoryImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public EObject create(EClass eClass) {
		switch (eClass.getClassifierID()) {
			case GMFGraphPackage.CANVAS: return createCanvas();
			case GMFGraphPackage.FIGURE_GALLERY: return createFigureGallery();
			case GMFGraphPackage.NODE: return createNode();
			case GMFGraphPackage.CONNECTION: return createConnection();
			case GMFGraphPackage.COMPARTMENT: return createCompartment();
			case GMFGraphPackage.DIAGRAM_LABEL: return createDiagramLabel();
			case GMFGraphPackage.GENERAL_FACET: return createGeneralFacet();
			case GMFGraphPackage.ALIGNMENT_FACET: return createAlignmentFacet();
			case GMFGraphPackage.GRADIENT_FACET: return createGradientFacet();
			case GMFGraphPackage.LABEL_OFFSET_FACET: return createLabelOffsetFacet();
			case GMFGraphPackage.DEFAULT_SIZE_FACET: return createDefaultSizeFacet();
			case GMFGraphPackage.FIGURE_DESCRIPTOR: return createFigureDescriptor();
			case GMFGraphPackage.CHILD_ACCESS: return createChildAccess();
			case GMFGraphPackage.FIGURE_REF: return createFigureRef();
			case GMFGraphPackage.LABEL: return createLabel();
			case GMFGraphPackage.LABELED_CONTAINER: return createLabeledContainer();
			case GMFGraphPackage.VERTICAL_LABEL: return createVerticalLabel();
			case GMFGraphPackage.RECTANGLE: return createRectangle();
			case GMFGraphPackage.INVISIBLE_RECTANGLE: return createInvisibleRectangle();
			case GMFGraphPackage.ROUNDED_RECTANGLE: return createRoundedRectangle();
			case GMFGraphPackage.ELLIPSE: return createEllipse();
			case GMFGraphPackage.POLYLINE: return createPolyline();
			case GMFGraphPackage.POLYGON: return createPolygon();
			case GMFGraphPackage.SCALABLE_POLYGON: return createScalablePolygon();
			case GMFGraphPackage.POLYLINE_CONNECTION: return createPolylineConnection();
			case GMFGraphPackage.POLYLINE_DECORATION: return createPolylineDecoration();
			case GMFGraphPackage.POLYGON_DECORATION: return createPolygonDecoration();
			case GMFGraphPackage.CUSTOM_ATTRIBUTE: return createCustomAttribute();
			case GMFGraphPackage.FIGURE_ACCESSOR: return createFigureAccessor();
			case GMFGraphPackage.CUSTOM_FIGURE: return createCustomFigure();
			case GMFGraphPackage.CUSTOM_DECORATION: return createCustomDecoration();
			case GMFGraphPackage.CUSTOM_CONNECTION: return createCustomConnection();
			case GMFGraphPackage.RGB_COLOR: return createRGBColor();
			case GMFGraphPackage.CONSTANT_COLOR: return createConstantColor();
			case GMFGraphPackage.BASIC_FONT: return createBasicFont();
			case GMFGraphPackage.POINT: return createPoint();
			case GMFGraphPackage.DIMENSION: return createDimension();
			case GMFGraphPackage.INSETS: return createInsets();
			case GMFGraphPackage.BORDER_REF: return createBorderRef();
			case GMFGraphPackage.LINE_BORDER: return createLineBorder();
			case GMFGraphPackage.MARGIN_BORDER: return createMarginBorder();
			case GMFGraphPackage.COMPOUND_BORDER: return createCompoundBorder();
			case GMFGraphPackage.CUSTOM_BORDER: return createCustomBorder();
			case GMFGraphPackage.CUSTOM_LAYOUT_DATA: return createCustomLayoutData();
			case GMFGraphPackage.GRID_LAYOUT_DATA: return createGridLayoutData();
			case GMFGraphPackage.BORDER_LAYOUT_DATA: return createBorderLayoutData();
			case GMFGraphPackage.LAYOUT_REF: return createLayoutRef();
			case GMFGraphPackage.CUSTOM_LAYOUT: return createCustomLayout();
			case GMFGraphPackage.GRID_LAYOUT: return createGridLayout();
			case GMFGraphPackage.BORDER_LAYOUT: return createBorderLayout();
			case GMFGraphPackage.FLOW_LAYOUT: return createFlowLayout();
			case GMFGraphPackage.XY_LAYOUT: return createXYLayout();
			case GMFGraphPackage.XY_LAYOUT_DATA: return createXYLayoutData();
			case GMFGraphPackage.STACK_LAYOUT: return createStackLayout();
			case GMFGraphPackage.CENTER_LAYOUT: return createCenterLayout();
			case GMFGraphPackage.SVG_FIGURE: return createSVGFigure();
			case GMFGraphPackage.SVG_PROPERTY: return createSVGProperty();
			case GMFGraphPackage.RECTANGLE2_D: return createRectangle2D();
			case GMFGraphPackage.CUSTOM_PIN: return createCustomPin();
			case GMFGraphPackage.COLOR_PIN: return createColorPin();
			case GMFGraphPackage.VISIBLE_PIN: return createVisiblePin();
			default:
				throw new IllegalArgumentException("The class '" + eClass.getName() + "' is not a valid classifier");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object createFromString(EDataType eDataType, String initialValue) {
		switch (eDataType.getClassifierID()) {
			case GMFGraphPackage.COLOR_CONSTANTS:
				return createColorConstantsFromString(eDataType, initialValue);
			case GMFGraphPackage.FONT_STYLE:
				return createFontStyleFromString(eDataType, initialValue);
			case GMFGraphPackage.DIRECTION:
				return createDirectionFromString(eDataType, initialValue);
			case GMFGraphPackage.LINE_KIND:
				return createLineKindFromString(eDataType, initialValue);
			case GMFGraphPackage.ALIGNMENT:
				return createAlignmentFromString(eDataType, initialValue);
			case GMFGraphPackage.SVG_PROPERTY_TYPE:
				return createSVGPropertyTypeFromString(eDataType, initialValue);
			default:
				throw new IllegalArgumentException("The datatype '" + eDataType.getName() + "' is not a valid classifier");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public String convertToString(EDataType eDataType, Object instanceValue) {
		switch (eDataType.getClassifierID()) {
			case GMFGraphPackage.COLOR_CONSTANTS:
				return convertColorConstantsToString(eDataType, instanceValue);
			case GMFGraphPackage.FONT_STYLE:
				return convertFontStyleToString(eDataType, instanceValue);
			case GMFGraphPackage.DIRECTION:
				return convertDirectionToString(eDataType, instanceValue);
			case GMFGraphPackage.LINE_KIND:
				return convertLineKindToString(eDataType, instanceValue);
			case GMFGraphPackage.ALIGNMENT:
				return convertAlignmentToString(eDataType, instanceValue);
			case GMFGraphPackage.SVG_PROPERTY_TYPE:
				return convertSVGPropertyTypeToString(eDataType, instanceValue);
			default:
				throw new IllegalArgumentException("The datatype '" + eDataType.getName() + "' is not a valid classifier");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Canvas createCanvas() {
		CanvasImpl canvas = new CanvasImpl();
		return canvas;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public FigureGallery createFigureGallery() {
		FigureGalleryImpl figureGallery = new FigureGalleryImpl();
		return figureGallery;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Node createNode() {
		NodeImpl node = new NodeImpl();
		return node;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public ChildAccess createChildAccess() {
		ChildAccessImpl childAccess = new ChildAccessImpl();
		return childAccess;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Connection createConnection() {
		ConnectionImpl connection = new ConnectionImpl();
		return connection;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Compartment createCompartment() {
		CompartmentImpl compartment = new CompartmentImpl();
		return compartment;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public DiagramLabel createDiagramLabel() {
		DiagramLabelImpl diagramLabel = new DiagramLabelImpl();
		return diagramLabel;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public GeneralFacet createGeneralFacet() {
		GeneralFacetImpl generalFacet = new GeneralFacetImpl();
		return generalFacet;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public AlignmentFacet createAlignmentFacet() {
		AlignmentFacetImpl alignmentFacet = new AlignmentFacetImpl();
		return alignmentFacet;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public GradientFacet createGradientFacet() {
		GradientFacetImpl gradientFacet = new GradientFacetImpl();
		return gradientFacet;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public LabelOffsetFacet createLabelOffsetFacet() {
		LabelOffsetFacetImpl labelOffsetFacet = new LabelOffsetFacetImpl();
		return labelOffsetFacet;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public DefaultSizeFacet createDefaultSizeFacet() {
		DefaultSizeFacetImpl defaultSizeFacet = new DefaultSizeFacetImpl();
		return defaultSizeFacet;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public FigureDescriptor createFigureDescriptor() {
		FigureDescriptorImpl figureDescriptor = new FigureDescriptorImpl();
		return figureDescriptor;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public FigureRef createFigureRef() {
		FigureRefImpl figureRef = new FigureRefImpl();
		return figureRef;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Label createLabel() {
		LabelImpl label = new LabelImpl();
		return label;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public LabeledContainer createLabeledContainer() {
		LabeledContainerImpl labeledContainer = new LabeledContainerImpl();
		return labeledContainer;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public VerticalLabel createVerticalLabel() {
		VerticalLabelImpl verticalLabel = new VerticalLabelImpl();
		return verticalLabel;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Rectangle createRectangle() {
		RectangleImpl rectangle = new RectangleImpl();
		return rectangle;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public InvisibleRectangle createInvisibleRectangle() {
		InvisibleRectangleImpl invisibleRectangle = new InvisibleRectangleImpl();
		return invisibleRectangle;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public RoundedRectangle createRoundedRectangle() {
		RoundedRectangleImpl roundedRectangle = new RoundedRectangleImpl();
		return roundedRectangle;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Ellipse createEllipse() {
		EllipseImpl ellipse = new EllipseImpl();
		return ellipse;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Polyline createPolyline() {
		PolylineImpl polyline = new PolylineImpl();
		return polyline;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Polygon createPolygon() {
		PolygonImpl polygon = new PolygonImpl();
		return polygon;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public ScalablePolygon createScalablePolygon() {
		ScalablePolygonImpl scalablePolygon = new ScalablePolygonImpl();
		return scalablePolygon;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public PolylineConnection createPolylineConnection() {
		PolylineConnectionImpl polylineConnection = new PolylineConnectionImpl();
		return polylineConnection;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public PolylineDecoration createPolylineDecoration() {
		PolylineDecorationImpl polylineDecoration = new PolylineDecorationImpl();
		return polylineDecoration;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public PolygonDecoration createPolygonDecoration() {
		PolygonDecorationImpl polygonDecoration = new PolygonDecorationImpl();
		return polygonDecoration;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public CustomFigure createCustomFigure() {
		CustomFigureImpl customFigure = new CustomFigureImpl();
		return customFigure;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public CustomDecoration createCustomDecoration() {
		CustomDecorationImpl customDecoration = new CustomDecorationImpl();
		return customDecoration;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public CustomConnection createCustomConnection() {
		CustomConnectionImpl customConnection = new CustomConnectionImpl();
		return customConnection;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public RGBColor createRGBColor() {
		RGBColorImpl rgbColor = new RGBColorImpl();
		return rgbColor;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public ConstantColor createConstantColor() {
		ConstantColorImpl constantColor = new ConstantColorImpl();
		return constantColor;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public BasicFont createBasicFont() {
		BasicFontImpl basicFont = new BasicFontImpl();
		return basicFont;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Point createPoint() {
		PointImpl point = new PointImpl();
		return point;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Dimension createDimension() {
		DimensionImpl dimension = new DimensionImpl();
		return dimension;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Insets createInsets() {
		InsetsImpl insets = new InsetsImpl();
		return insets;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public BorderRef createBorderRef() {
		BorderRefImpl borderRef = new BorderRefImpl();
		return borderRef;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public LineBorder createLineBorder() {
		LineBorderImpl lineBorder = new LineBorderImpl();
		return lineBorder;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public MarginBorder createMarginBorder() {
		MarginBorderImpl marginBorder = new MarginBorderImpl();
		return marginBorder;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public CompoundBorder createCompoundBorder() {
		CompoundBorderImpl compoundBorder = new CompoundBorderImpl();
		return compoundBorder;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public CustomBorder createCustomBorder() {
		CustomBorderImpl customBorder = new CustomBorderImpl();
		return customBorder;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public CustomLayoutData createCustomLayoutData() {
		CustomLayoutDataImpl customLayoutData = new CustomLayoutDataImpl();
		return customLayoutData;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public GridLayoutData createGridLayoutData() {
		GridLayoutDataImpl gridLayoutData = new GridLayoutDataImpl();
		return gridLayoutData;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public BorderLayoutData createBorderLayoutData() {
		BorderLayoutDataImpl borderLayoutData = new BorderLayoutDataImpl();
		return borderLayoutData;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public LayoutRef createLayoutRef() {
		LayoutRefImpl layoutRef = new LayoutRefImpl();
		return layoutRef;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public CustomLayout createCustomLayout() {
		CustomLayoutImpl customLayout = new CustomLayoutImpl();
		return customLayout;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public GridLayout createGridLayout() {
		GridLayoutImpl gridLayout = new GridLayoutImpl();
		return gridLayout;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public BorderLayout createBorderLayout() {
		BorderLayoutImpl borderLayout = new BorderLayoutImpl();
		return borderLayout;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public CustomAttribute createCustomAttribute() {
		CustomAttributeImpl customAttribute = new CustomAttributeImpl();
		return customAttribute;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public FigureAccessor createFigureAccessor() {
		FigureAccessorImpl figureAccessor = new FigureAccessorImpl();
		return figureAccessor;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public FlowLayout createFlowLayout() {
		FlowLayoutImpl flowLayout = new FlowLayoutImpl();
		return flowLayout;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public XYLayout createXYLayout() {
		XYLayoutImpl xyLayout = new XYLayoutImpl();
		return xyLayout;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public XYLayoutData createXYLayoutData() {
		XYLayoutDataImpl xyLayoutData = new XYLayoutDataImpl();
		return xyLayoutData;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public StackLayout createStackLayout() {
		StackLayoutImpl stackLayout = new StackLayoutImpl();
		return stackLayout;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public CenterLayout createCenterLayout() {
		CenterLayoutImpl centerLayout = new CenterLayoutImpl();
		return centerLayout;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public SVGFigure createSVGFigure() {
		SVGFigureImpl svgFigure = new SVGFigureImpl();
		return svgFigure;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public SVGProperty createSVGProperty() {
		SVGPropertyImpl svgProperty = new SVGPropertyImpl();
		return svgProperty;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Rectangle2D createRectangle2D() {
		Rectangle2DImpl rectangle2D = new Rectangle2DImpl();
		return rectangle2D;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public CustomPin createCustomPin() {
		CustomPinImpl customPin = new CustomPinImpl();
		return customPin;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public ColorPin createColorPin() {
		ColorPinImpl colorPin = new ColorPinImpl();
		return colorPin;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public VisiblePin createVisiblePin() {
		VisiblePinImpl visiblePin = new VisiblePinImpl();
		return visiblePin;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public ColorConstants createColorConstantsFromString(EDataType eDataType, String initialValue) {
		ColorConstants result = ColorConstants.get(initialValue);
		if (result == null) throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '" + eDataType.getName() + "'");
		return result;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public String convertColorConstantsToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public FontStyle createFontStyleFromString(EDataType eDataType, String initialValue) {
		FontStyle result = FontStyle.get(initialValue);
		if (result == null) throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '" + eDataType.getName() + "'");
		return result;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public String convertFontStyleToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Direction createDirectionFromString(EDataType eDataType, String initialValue) {
		Direction result = Direction.get(initialValue);
		if (result == null) throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '" + eDataType.getName() + "'");
		return result;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public String convertDirectionToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public LineKind createLineKindFromString(EDataType eDataType, String initialValue) {
		LineKind result = LineKind.get(initialValue);
		if (result == null) throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '" + eDataType.getName() + "'");
		return result;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public String convertLineKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Alignment createAlignmentFromString(EDataType eDataType, String initialValue) {
		Alignment result = Alignment.get(initialValue);
		if (result == null) throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '" + eDataType.getName() + "'");
		return result;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public String convertAlignmentToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public SVGPropertyType createSVGPropertyTypeFromString(EDataType eDataType, String initialValue) {
		SVGPropertyType result = SVGPropertyType.get(initialValue);
		if (result == null) throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '" + eDataType.getName() + "'");
		return result;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public String convertSVGPropertyTypeToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public GMFGraphPackage getGMFGraphPackage() {
		return (GMFGraphPackage)getEPackage();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @deprecated
	 * @generated
	 */
	@Deprecated
	public static GMFGraphPackage getPackage() {
		return GMFGraphPackage.eINSTANCE;
	}

} //GMFGraphFactoryImpl
