/**
 * Copyright (c) 2012-2020 Montages AG, CEA LIST, Artal
 * 
 * All rights reserved. This program && the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, && is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *    Svyatoslav Kovalsky - initial API and implementation
 *    Michael Golubev (Borland) - #386838 - migrate to Xtend2
 *    Aurelien Didier (ARTAL) - aurelien.didier51@gmail.com - Bug 569174
 */
package xpt.diagram.editpolicies;

import com.google.common.base.Objects;
import com.google.inject.Inject;
import com.google.inject.Singleton;
import java.util.Arrays;
import metamodel.MetaModel;
import org.eclipse.emf.codegen.ecore.genmodel.GenClass;
import org.eclipse.emf.codegen.ecore.genmodel.GenFeature;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.ocl.ecore.PrimitiveType;
import org.eclipse.ocl.ecore.TupleType;
import org.eclipse.papyrus.gmf.codegen.gmfgen.FeatureLinkModelFacet;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenCommonBase;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenLink;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenVisualEffect;
import org.eclipse.papyrus.gmf.codegen.gmfgen.LinkModelFacet;
import org.eclipse.papyrus.gmf.codegen.gmfgen.TypeLinkModelFacet;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Extension;
import xpt.CodeStyle;
import xpt.Common;
import xpt.Common_qvto;
import xpt.diagram.editparts.EditPartFactory;
import xpt.expressions.OclTracker_qvto;

@Singleton
@SuppressWarnings("all")
public class VisualEffectEditPolicy {
  @Inject
  @Extension
  private Common _common;
  
  @Inject
  @Extension
  private Common_qvto _common_qvto;
  
  @Inject
  @Extension
  private OclTracker_qvto _oclTracker_qvto;
  
  @Inject
  private MetaModel xptMetaModel;
  
  @Inject
  private CodeStyle xptCodeStyle;
  
  @Inject
  private EditPartFactory xptEditPartFactory;
  
  public CharSequence className(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    String _lastSegment = this._common_qvto.lastSegment(it.getEditPolicyQualifiedClassName());
    _builder.append(_lastSegment);
    return _builder;
  }
  
  public CharSequence packageName(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    String _withoutLastSegment = this._common_qvto.withoutLastSegment(it.getEditPolicyQualifiedClassName());
    _builder.append(_withoutLastSegment);
    return _builder;
  }
  
  public CharSequence qualifiedClassName(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _packageName = this.packageName(it);
    _builder.append(_packageName);
    _builder.append(".");
    CharSequence _className = this.className(it);
    _builder.append(_className);
    return _builder;
  }
  
  public CharSequence fullPath(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _qualifiedClassName = this.qualifiedClassName(it);
    _builder.append(_qualifiedClassName);
    return _builder;
  }
  
  public CharSequence VisualEffectEditPolicy(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _copyright = this._common.copyright(it.getSubject().getDiagram().getEditorGen());
    _builder.append(_copyright);
    _builder.newLineIfNotEmpty();
    _builder.append("package ");
    CharSequence _packageName = this.packageName(it);
    _builder.append(_packageName);
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _generatedClassComment = this._common.generatedClassComment();
    _builder.append(_generatedClassComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    CharSequence _className = this.className(it);
    _builder.append(_className);
    _builder.append(" ");
    CharSequence _extendsList = this.extendsList(it);
    _builder.append(_extendsList);
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _fields = this.fields(it);
    _builder.append(_fields, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _constructor = this.constructor(it);
    _builder.append(_constructor, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _setVisualEffectValue = this.setVisualEffectValue(it);
    _builder.append(_setVisualEffectValue, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _hostImpl = this.getHostImpl(it);
    _builder.append(_hostImpl, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _expressionBody = this.getExpressionBody(it);
    _builder.append(_expressionBody, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _context = this.getContext(it);
    _builder.append(_context, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _additions = this.additions(it);
    _builder.append(_additions, "\t");
    _builder.append("\t");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence extendsList(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("extends org.eclipse.gmf.tooling.runtime.edit.policies.effect.AbstractOclVisualEffectEditPolicy");
    return _builder;
  }
  
  public CharSequence fields(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public static final String KEY = \"");
    CharSequence _qualifiedClassName = this.qualifiedClassName(it);
    _builder.append(_qualifiedClassName);
    _builder.append(":KEY\";");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public CharSequence constructor(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public ");
    CharSequence _className = this.className(it);
    _builder.append(_className);
    _builder.append("() {");
    _builder.newLineIfNotEmpty();
    {
      boolean _isForcedImpactAnalyzerKind = this._oclTracker_qvto.isForcedImpactAnalyzerKind(it.getOclExpression());
      if (_isForcedImpactAnalyzerKind) {
        _builder.append("super(org.eclipse.gmf.tooling.runtime.ocl.tracker.OclTrackerFactory.Type.IMPACT_ANALYZER);");
        _builder.newLine();
      } else {
        _builder.append("super();");
        _builder.newLine();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence setVisualEffectValue(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _overrideAnnotationC = this.overrideAnnotationC(it);
    _builder.append(_overrideAnnotationC);
    _builder.newLineIfNotEmpty();
    _builder.append("protected void setVisualEffectValue(Object value) {");
    _builder.newLine();
    {
      String _pinKind = it.getPinKind();
      boolean _equals = Objects.equal(_pinKind, "ColorPin");
      if (_equals) {
        _builder.append("\t");
        CharSequence _setVisualEffectValueOnColorPin = this.setVisualEffectValueOnColorPin(it);
        _builder.append(_setVisualEffectValueOnColorPin, "\t");
        _builder.newLineIfNotEmpty();
      } else {
        String _pinKind_1 = it.getPinKind();
        boolean _equals_1 = Objects.equal(_pinKind_1, "VisiblePin");
        if (_equals_1) {
          _builder.append("\t");
          CharSequence _setVisualEffectValueOnCustomPin = this.setVisualEffectValueOnCustomPin(it);
          _builder.append(_setVisualEffectValueOnCustomPin, "\t");
          _builder.newLineIfNotEmpty();
        } else {
          String _pinKind_2 = it.getPinKind();
          boolean _equals_2 = Objects.equal(_pinKind_2, "CustomPin");
          if (_equals_2) {
            _builder.append("\t");
            CharSequence _setVisualEffectValueOnCustomPin_1 = this.setVisualEffectValueOnCustomPin(it);
            _builder.append(_setVisualEffectValueOnCustomPin_1, "\t");
            _builder.newLineIfNotEmpty();
          } else {
            _builder.append("\t");
            String _pinKind_3 = it.getPinKind();
            String _plus = ("Unsupported Pin kind: " + _pinKind_3);
            this._common_qvto.ERROR(_plus);
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getHostImpl(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    CharSequence _overrideAnnotationC = this.overrideAnnotationC(it);
    _builder.append(_overrideAnnotationC);
    _builder.newLineIfNotEmpty();
    _builder.append("protected ");
    CharSequence _editPartQualifiedClassName = this.xptEditPartFactory.getEditPartQualifiedClassName(it.getSubject());
    _builder.append(_editPartQualifiedClassName);
    _builder.append(" getHostImpl() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return (");
    CharSequence _editPartQualifiedClassName_1 = this.xptEditPartFactory.getEditPartQualifiedClassName(it.getSubject());
    _builder.append(_editPartQualifiedClassName_1, "\t");
    _builder.append(") super.getHostImpl();");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getExpressionBody(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    CharSequence _overrideAnnotationC = this.overrideAnnotationC(it);
    _builder.append(_overrideAnnotationC);
    _builder.newLineIfNotEmpty();
    _builder.append("protected String getExpressionBody() {\t");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return ");
    _builder.newLine();
    {
      boolean _oclIsKindOf = this._common_qvto.oclIsKindOf(it.getSubject(), GenLink.class);
      if (_oclIsKindOf) {
        GenCommonBase _subject = it.getSubject();
        CharSequence _expressionLinkEnds = this.expressionLinkEnds(((GenLink) _subject).getModelFacet());
        _builder.append(_expressionLinkEnds);
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t       ");
    String _oclExpressionString = it.getOclExpressionString();
    _builder.append(_oclExpressionString, "\t       ");
    _builder.append("; ");
    CharSequence _nonNLS = this._common.nonNLS(1);
    _builder.append(_nonNLS, "\t       ");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence _expressionLinkEnds(final LinkModelFacet it) {
    StringConcatenation _builder = new StringConcatenation();
    return _builder;
  }
  
  protected CharSequence _expressionLinkEnds(final FeatureLinkModelFacet it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      int _upperBound = it.getMetaFeature().getEcoreFeature().getUpperBound();
      boolean _equals = (_upperBound == 1);
      if (_equals) {
        String _name = it.getMetaFeature().getEcoreFeature().getName();
        String _plus = ("." + _name);
        CharSequence _expressionLinkEnds = this.expressionLinkEnds(it, "", _plus);
        _builder.append(_expressionLinkEnds);
        _builder.newLineIfNotEmpty();
      } else {
        {
          GenFeature _reverse = it.getMetaFeature().getReverse();
          boolean _notEquals = (!Objects.equal(_reverse, null));
          if (_notEquals) {
            String _name_1 = it.getMetaFeature().getReverse().getEcoreFeature().getName();
            String _plus_1 = ("." + _name_1);
            CharSequence _expressionLinkEnds_1 = this.expressionLinkEnds(it, _plus_1, "");
            _builder.append(_expressionLinkEnds_1);
            _builder.append("\t\t\t\t");
            _builder.newLineIfNotEmpty();
          } else {
            EStructuralFeature _ecoreFeature = it.getMetaFeature().getEcoreFeature();
            boolean _isContainment = ((EReference) _ecoreFeature).isContainment();
            if (_isContainment) {
              String _name_2 = it.getSourceType().getEcoreClass().getName();
              String _plus_2 = (".oclAsType(ecore::EObject).eContainer().oclAsType(" + _name_2);
              String _plus_3 = (_plus_2 + ")");
              CharSequence _expressionLinkEnds_2 = this.expressionLinkEnds(it, _plus_3, "");
              _builder.append(_expressionLinkEnds_2);
              _builder.newLineIfNotEmpty();
            } else {
              _builder.append("/* unable to set up both link ends */\t\t\t");
              _builder.newLine();
            }
          }
        }
      }
    }
    return _builder;
  }
  
  protected CharSequence _expressionLinkEnds(final TypeLinkModelFacet it) {
    StringConcatenation _builder = new StringConcatenation();
    String _name = it.getSourceMetaFeature().getEcoreFeature().getName();
    String _plus = ("." + _name);
    String _name_1 = it.getTargetMetaFeature().getEcoreFeature().getName();
    String _plus_1 = ("." + _name_1);
    CharSequence _expressionLinkEnds = this.expressionLinkEnds(it, _plus, _plus_1);
    _builder.append(_expressionLinkEnds);
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public CharSequence expressionLinkEnds(final LinkModelFacet it, final String sourcePath, final String targetPath) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("\" let _src : ");
    String _name = it.getSourceType().getEcoreClass().getName();
    _builder.append(_name);
    _builder.append(" = self");
    _builder.append(sourcePath);
    _builder.append(" in \" //");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("+ \" let _dst : ");
    String _name_1 = it.getTargetType().getEcoreClass().getName();
    _builder.append(_name_1, "\t\t");
    _builder.append(" = self");
    _builder.append(targetPath, "\t\t");
    _builder.append(" in \" //");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("+");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence callOperation(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("getHostImpl().getPrimaryShape().");
    String _operationName = it.getOperationName();
    _builder.append(_operationName);
    return _builder;
  }
  
  public CharSequence setVisualEffectValueOnColorPin(final GenVisualEffect it) {
    CharSequence _xblockexpression = null;
    {
      EClassifier _operationRuntimeType = it.getOperationRuntimeType();
      TupleType tupleType = ((TupleType) _operationRuntimeType);
      StringConcatenation _builder = new StringConcatenation();
      CharSequence _defineTupleParts = this.defineTupleParts(tupleType);
      _builder.append(_defineTupleParts);
      _builder.newLineIfNotEmpty();
      _builder.append("org.eclipse.swt.graphics.Color color = new org.eclipse.swt.graphics.Color(null, ");
      CharSequence _enumerateTupleParts = this.enumerateTupleParts(tupleType);
      _builder.append(_enumerateTupleParts);
      _builder.append(");\t\t\t");
      _builder.newLineIfNotEmpty();
      CharSequence _callOperation = this.callOperation(it);
      _builder.append(_callOperation);
      _builder.append("(color);");
      _builder.newLineIfNotEmpty();
      _xblockexpression = _builder;
    }
    return _xblockexpression;
  }
  
  public CharSequence setVisualEffectValueOnCustomPin(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _setVisualEffectValueOfType = this.setVisualEffectValueOfType(it.getOperationRuntimeType(), it);
    _builder.append(_setVisualEffectValueOfType);
    _builder.newLineIfNotEmpty();
    CharSequence _extraLineBreak = this._common.extraLineBreak();
    _builder.append(_extraLineBreak);
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _setVisualEffectValueOfType(final EClassifier it, final GenVisualEffect visualEffect) {
    StringConcatenation _builder = new StringConcatenation();
    this._common_qvto.ERROR(("Abstract template call setVisualEffectValueOfType for " + it));
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _setVisualEffectValueOfType(final PrimitiveType it, final GenVisualEffect visualEffect) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _callOperation = this.callOperation(visualEffect);
    _builder.append(_callOperation);
    _builder.append("((");
    String _instanceTypeName = it.getInstanceTypeName();
    _builder.append(_instanceTypeName);
    _builder.append(")value);");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  protected CharSequence _setVisualEffectValueOfType(final TupleType it, final GenVisualEffect visualEffect) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _defineTupleParts = this.defineTupleParts(it);
    _builder.append(_defineTupleParts);
    _builder.newLineIfNotEmpty();
    CharSequence _extraLineBreak = this._common.extraLineBreak();
    _builder.append(_extraLineBreak);
    _builder.newLineIfNotEmpty();
    CharSequence _callOperation = this.callOperation(visualEffect);
    _builder.append(_callOperation);
    _builder.newLineIfNotEmpty();
    _builder.append("(");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _enumerateTupleParts = this.enumerateTupleParts(it);
    _builder.append(_enumerateTupleParts, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append(");\t");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence defineTupleParts(final TupleType it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("org.eclipse.ocl.util.Tuple<?, ?> tupleValue = (org.eclipse.ocl.util.Tuple<?, ?>) value;");
    _builder.newLine();
    {
      EList<EAttribute> _eAllAttributes = it.getEAllAttributes();
      for(final EAttribute attribute : _eAllAttributes) {
        String _instanceTypeName = attribute.getEType().getInstanceTypeName();
        _builder.append(_instanceTypeName);
        _builder.append(" ");
        String _name = attribute.getName();
        _builder.append(_name);
        _builder.append(" = (");
        String _instanceTypeName_1 = attribute.getEType().getInstanceTypeName();
        _builder.append(_instanceTypeName_1);
        _builder.append(")tupleValue.getValue(\"");
        String _name_1 = attribute.getName();
        _builder.append(_name_1);
        _builder.append("\");");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence enumerateTupleParts(final TupleType it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<EAttribute> _eAllAttributes = it.getEAllAttributes();
      boolean _hasElements = false;
      for(final EAttribute attribute : _eAllAttributes) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "");
        }
        String _name = attribute.getName();
        _builder.append(_name);
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence getContext(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _oclIsKindOf = this._common_qvto.oclIsKindOf(it.getSubject(), GenLink.class);
      if (_oclIsKindOf) {
        GenCommonBase _subject = it.getSubject();
        CharSequence _facetContext = this.getFacetContext(((GenLink) _subject).getModelFacet());
        _builder.append(_facetContext);
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  protected CharSequence _getFacetContext(final LinkModelFacet it) {
    StringConcatenation _builder = new StringConcatenation();
    return _builder;
  }
  
  protected CharSequence _getFacetContext(final FeatureLinkModelFacet it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    EObject _eContainer = it.eContainer();
    CharSequence _overrideC = this.xptCodeStyle.overrideC(((GenCommonBase) _eContainer));
    _builder.append(_overrideC);
    _builder.newLineIfNotEmpty();
    _builder.append("protected org.eclipse.emf.ecore.EObject getContext() {");
    _builder.newLine();
    {
      int _upperBound = it.getMetaFeature().getEcoreFeature().getUpperBound();
      boolean _equals = (_upperBound == 1);
      if (_equals) {
        CharSequence _contextBody = this.getContextBody(it, "source", "target", it.getSourceType(), it.getTargetType(), it.getMetaFeature());
        _builder.append(_contextBody);
        _builder.newLineIfNotEmpty();
      } else {
        {
          if (((!Objects.equal(it.getMetaFeature().getReverse(), null)) || ((EReference) it.getMetaFeature().getEcoreFeature()).isContainment())) {
            CharSequence _contextBody_1 = this.getContextBody(it, "target", "source", it.getTargetType(), it.getSourceType(), it.getMetaFeature().getReverse());
            _builder.append(_contextBody_1);
            _builder.newLineIfNotEmpty();
          } else {
            _builder.append("// unable to pass both link ends ");
            _builder.newLine();
            _builder.append("return null;\t\t\t");
            _builder.newLine();
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getContextBody(final FeatureLinkModelFacet it, final String selfName, final String oppositeName, final GenClass selfType, final GenClass oppositeType, final GenFeature selfFeature) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("org.eclipse.gmf.runtime.notation.Edge hostModel = (org.eclipse.gmf.runtime.notation.Edge) host().getModel();");
    _builder.newLine();
    _builder.append("org.eclipse.gmf.runtime.notation.View targetModel = hostModel.getTarget();");
    _builder.newLine();
    _builder.append("org.eclipse.gmf.runtime.notation.View sourceModel = hostModel.getSource();");
    _builder.newLine();
    _builder.newLine();
    _builder.append("if (targetModel == null || sourceModel == null) { ");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return null;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("org.eclipse.emf.ecore.EObject ");
    _builder.append(selfName);
    _builder.append("SemanticModel = org.eclipse.gmf.runtime.diagram.core.util.ViewUtil.resolveSemanticElement(");
    _builder.append(selfName);
    _builder.append("Model);");
    _builder.newLineIfNotEmpty();
    _builder.append("// need to check actual opposite of the semantic element, since diagram opposite could be not up to date");
    _builder.newLine();
    {
      boolean _notEquals = (!Objects.equal(selfFeature, null));
      if (_notEquals) {
        CharSequence _QualifiedClassName = this.xptMetaModel.QualifiedClassName(selfType);
        _builder.append(_QualifiedClassName);
        _builder.append(" ");
        _builder.append(selfName);
        _builder.append("TypedModel = ");
        CharSequence _CastEObject = this.xptMetaModel.CastEObject(selfType, 
          (selfName + "SemanticModel"));
        _builder.append(_CastEObject);
        _builder.append(";");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("org.eclipse.emf.ecore.EObject ");
    _builder.append(oppositeName);
    _builder.append("SemanticModel =");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    {
      boolean _notEquals_1 = (!Objects.equal(selfFeature, null));
      if (_notEquals_1) {
        CharSequence _featureValue = this.xptMetaModel.getFeatureValue(selfFeature, (selfName + "TypedModel"), oppositeType);
        _builder.append(_featureValue, "\t");
      } else {
        _builder.append(selfName, "\t");
        _builder.append("SemanticModel.eContainer()");
      }
    }
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("if (");
    _builder.append(oppositeName);
    _builder.append("SemanticModel == null) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return null;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("return ");
    _builder.append(selfName);
    _builder.append("SemanticModel; ");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public CharSequence overrideAnnotationC(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    EObject _eContainer = it.eContainer();
    CharSequence _overrideC = this.xptCodeStyle.overrideC(((GenCommonBase) _eContainer));
    _builder.append(_overrideC);
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public CharSequence additions(final GenVisualEffect it) {
    StringConcatenation _builder = new StringConcatenation();
    return _builder;
  }
  
  public CharSequence expressionLinkEnds(final LinkModelFacet it) {
    if (it instanceof FeatureLinkModelFacet) {
      return _expressionLinkEnds((FeatureLinkModelFacet)it);
    } else if (it instanceof TypeLinkModelFacet) {
      return _expressionLinkEnds((TypeLinkModelFacet)it);
    } else if (it != null) {
      return _expressionLinkEnds(it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
  
  public CharSequence setVisualEffectValueOfType(final EClassifier it, final GenVisualEffect visualEffect) {
    if (it instanceof PrimitiveType) {
      return _setVisualEffectValueOfType((PrimitiveType)it, visualEffect);
    } else if (it instanceof TupleType) {
      return _setVisualEffectValueOfType((TupleType)it, visualEffect);
    } else if (it != null) {
      return _setVisualEffectValueOfType(it, visualEffect);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it, visualEffect).toString());
    }
  }
  
  public CharSequence getFacetContext(final LinkModelFacet it) {
    if (it instanceof FeatureLinkModelFacet) {
      return _getFacetContext((FeatureLinkModelFacet)it);
    } else if (it != null) {
      return _getFacetContext(it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
}
